@@ -37,6 +37,8 @@ class Repo(SourceBaseCommand):
     ['tarball'] (optional):         The tarball base to accelerate the fetch.
     ['repo_downloads'] (optional):  Repo downloads to do. Computer from GerritChangeSource
                                     and forced build properties.
+    ['jobs'] (optional):            number of connections to run in parallel
+                                    repo tool will use while syncing
     """
 
     header = "repo operation"
@@ -52,8 +54,9 @@ class Repo(SourceBaseCommand):
         # we're using string instead of an array here, because it will be transferred back
         # to the master as string anyway and using eval() could have security implications.
         self.repo_downloaded = ""
+        self.jobs = args.get('jobs')
 
-        self.sourcedata = "%s %s %s" % (self.manifest_url, self.manifest_branch, self.manifest_file)
+        self.sourcedata = "%s %s" % (self.manifest_url, self.manifest_file)
         self.re_change = re.compile(".* refs/changes/\d\d/(\d+)/(\d+) -> FETCH_HEAD$")
         self.re_head = re.compile("^HEAD is now at ([0-9a-f]+)...")
 
@@ -137,12 +140,12 @@ class Repo(SourceBaseCommand):
                 ln -sf manifests/%(manifest_file)s manifest.xml
                 cd ..
              fi
-             find . -name .git/index.lock -exec rm -f {} \;
+             repo forall -c rm -f .git/index.lock
                repo forall -c git clean -f -d -x 2>/dev/null
                 repo forall -c git reset --hard HEAD 2>/dev/null
              """) % self.__dict__
         return "\n".join([ s.strip() for s in command.splitlines()])
-    
+
     def _doPreInitCleanUp(self, dummy):
         command = self._cleanupCommand()
         return self._Cmd(["bash", "-c", command], self._doInit, abandonOnFailure=False)
@@ -165,6 +168,8 @@ class Repo(SourceBaseCommand):
         if self.manifest_override_url:
             os.system("cd %s/.repo; ln -sf ../manifest_override.xml manifest.xml"%(self._fullSrcdir()))
         command = ['sync']
+        if self.jobs:
+          command.append('-j' + str(self.jobs))
         self.sendStatus({"header": "synching manifest %s from branch %s from %s\n"
                                    % (self.manifest_file, self.manifest_branch, self.manifest_url)})
         return self._repoCmd(command, self._didSync)
